<?php
/**
* 2007-2018 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
*
*  @author    Hadi Mollaei <mr.hadimollaei@gmail.com>
*  @copyright 2007-2018 Rangine.ir
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/

if (!defined('_PS_VERSION_')) {
    exit;
}
class RangineSmsPresta extends Module
{
    private $prefix;
    public function __construct()
    {
        $this->preload();
        $this->name          = 'ranginesmspresta';
        $this->tab           = 'emailing';
        $this->version       = '2.0.2';
        $this->author        = 'Hadi Mollaei';
        $this->support        = 'http://rangine.ir/';
        $this->need_instance = 0;
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName      = $this->l('Rangine Sms Gateway');
        $this->description      = $this->l('Rangine sms web service for PrestaShop');
        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');
        $this->prefix           = 'RANGINE_SMS_';
		$context = Context::getContext();
		$this->shop_id = (int)$context->shop->id;
        $this->shop = $context->shop->name;
        $this->enable            = Configuration::get($this->prefix .'SMSENABLE' , null , null , $this->shop_id);
        $this->PSversion = Configuration::get('PS_INSTALL_VERSION');
        $this->domain = Configuration::get('PS_SHOP_DOMAIN');
        $this->license = Configuration::get($this->prefix .'LICENSE');
        // Some long messages that make the code messy. TODO: May I can bring all messages here?!
        $this->text = array(
            'NEWACADTEXT' => $this->l('{firstname} {lastname} registred on site'),
            'NEWACCUTEXT' => $this->l('Dear {firstname} {lastname},~Welcome to our site.'),
            'NEWORCUTEXT' => $this->l('Dear {firstname} {lastname},~Your order has created.~Order No: {order_id}~{shop_name}'),
            'UPORCUTEXT' => $this->l('{shop_name}~Dear {firstname} {lastname},~New status for order: {order_id}: {order_state}'),
            'UPORTRTEXT' => $this->l('Dear {firstname} {lastname},~Your order has sent by {carrier}~Order: {order_id}~Tracking No: {tracking}~{shop_name}'),
            'NewOrderTextDesc' => $this->l('Variables: {firstname} {lastname} {order_id} {payment} {total_paid} {currency}'),
            'NewOrderCustomerTextDesc' => $this->l('Variables: {firstname} {lastname} {order_id} {payment} {total_paid} {currency}'),
            'UpdateOrderTrackingHint' => $this->l('Select which type you want to send to customer on update order tracking number.'),
            'AdminOutOfStockTypeHint' => $this->l('Select which type you want to send to admin when a product goes out of stock.'),
            'AdminBackToStockTypeHint' => $this->l('Select which type you want to send to subscribers when a product goes back to stock.'),
            'INVOICEDESC' => $this->l('Send generated short link of Invoice at end of sms text (max: 30 character, like: Invoice: https://goo.gl/XXXXXX'),
			'prosign' => '<span class="pro-version licensed" title="'.$this->l('Only in Licensed Version').'">pro</span>',
        );
		if($this->ml() != 'true') $this->text['prosign'] = '<span class="pro-version" title="'.$this->l('Only in Licensed Version').'">pro</span>';
    }
	
	/***
	 * Install module
	 */
    public function install()
    {
		if (Shop::isFeatureActive())
		{
		  Shop::setContext(Shop::CONTEXT_ALL);
		}
        if (!extension_loaded('curl')) {
            return false;
        }
        if (!$this->installConfig()) {
            return false;
        }
        if (!$this->installDatabase()) {
			
            return false;
        }
		if(!parent::install()
			or !$this->installModuleTab($this->name, array(1 => $this->displayName), 0)) return false;
		include_once('include/hooks.php');
		foreach($this->hooks as $hook){
			if (!$this->registerHook($hook)) $error = 1;
        }
		if($error == 1) return false; else return true;
    }

	/***
	 * Uninstall module
	 */
    public function uninstall()
    {
		if(Configuration::get($this->prefix .'UNINSTALLDB' , null , null , null) == '1'){
			if (!$this->unistallDatabase()) { 
				return false;
			} 
		}
        if (!$this->uninstallConfig()) {
            return false;
        }
        if (!parent::uninstall()) {
            return false;
        }
        if (!$this->uninstallModuleTab($this->name)) {
            return false;
        }

        return true;
    }

	/***
	 * Install Configs in prestashop configuration table 
	 */
    protected function installConfig()
    {
		include_once('include/configs.php');
		foreach($this->configs as $key => $value){
			if (!Configuration::updateValue($this->prefix . $key, $value)) return false;
        }
        return true;
    }

	/***
	 * Remove module configs from prestashop configuration table 
	 */
    protected function uninstallConfig()
    {
		include_once('include/configs.php');
		foreach($this->configs as $key => $value){
			if (!Configuration::deleteByName($this->prefix . $key)) return false;
        }
        return true;
    }

	/***
	 * Install SMS logs table 
	 */
    protected function installDatabase()
    {
        $sql = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'ranginesmspresta`(
                `id_ranginesmspresta` INT(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,
                `customer` INT(5) DEFAULT NULL ,
                `phone` VARCHAR(100) DEFAULT NULL ,
                `position` VARCHAR(50) DEFAULT NULL ,
                `status` VARCHAR(15) NOT NULL ,
                `bulk` varchar(50) DEFAULT NULL,
                `delivery` varchar(50) DEFAULT NULL,
                `description` TEXT DEFAULT NULL,
				`timestamp` varchar(15) DEFAULT NULL,
				`type` varchar(10) DEFAULT NULL,
				`shop` varchar(10) DEFAULT NULL
                 )ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8;';
        $sql .= "CREATE TABLE IF NOT EXISTS `" . _DB_PREFIX_ . "ranginesmspresta_oos`(
			  `id_customer` int(10) unsigned DEFAULT '0',
			  `phone_number` varchar(15) NOT NULL,
			  `id_product` int(10) unsigned NOT NULL,
			  `id_product_attribute` int(10) unsigned NOT NULL,
			  `id_shop` int(10) unsigned NOT NULL,
			  `id_lang` int(10) unsigned NOT NULL,
			  `timestamp` varchar(10) DEFAULT NULL,
			  PRIMARY KEY (`phone_number`,`id_product`,`id_product_attribute`,`id_shop`)
			) ENGINE=" . _MYSQL_ENGINE_ . " DEFAULT CHARSET=utf8;";
        return Db::getInstance()->execute($sql);
    }
	
	/***
	 * Drop SMS logs table 
	 */
    protected function unistallDatabase()
    {
        $sql = 'DROP TABLE IF EXISTS `' . _DB_PREFIX_ . 'ranginesmspresta`;';
        $sql .= 'DROP TABLE IF EXISTS `' . _DB_PREFIX_ . 'ranginesmspresta_oos`;';
        return Db::getInstance()->execute($sql);
    }
	
	/**
	 * Clear data from database.
	 *
	 */
    protected function emptyDatabase()
    {
        $sql = 'TRUNCATE TABLE `' . _DB_PREFIX_ . $this->name . '`';
        return Db::getInstance()->execute($sql);
    }
	
    /**
     * Create plugin tab. Called from install function.
     *
     * @param string $tabClass
     * @param string $tabName
     * @param array $idTabParent
     * @return bool
     */
    private function installModuleTab($tabClass, $tabName, $idTabParent)
    {
        $partab             = new Tab();
        $partab->name       = $tabName;
        $partab->class_name = $tabClass;
        $partab->module     = $this->name;
        $partab->id_parent  = $idTabParent;
        if ($partab->save()) {
            return true;
        }
        return false;
    }

    /**
     * Delete plugin tab. Called from uninstall function.
     *
     * @param string $tabClass
     * @return bool
     */
    private function uninstallModuleTab($tabClass)
    {
        $sql = 'SELECT id_tab FROM ' . _DB_PREFIX_ . 'tab WHERE class_name = "' . pSQL($tabClass) . '"';
        if ($results = Db::getInstance()->ExecuteS($sql)) {
            foreach ($results as $row) {
                $idTab = $row['id_tab'];
                if ($idTab != 0) {
                    $tab = new Tab((int)$idTab);
                    $tab->delete();
                }
            }
        }
        return true;
    }
	
    /**
     * Initialization function called from Constructor.
	 * Redirect to module configure page when click on module tab
     */
    public function preload()
    {
        if (Tools::getValue('controller') != ''
            && Tools::getValue('controller') == 'ranginesmspresta') {
                $token   = Tools::getAdminTokenLite('AdminModules');
                if(isset($_SERVER['REQUEST_SCHEME']) && $_SERVER['REQUEST_SCHEME']) $request_scheme = $_SERVER['REQUEST_SCHEME']; else $request_scheme = 'http';
                $hostlink = $request_scheme . "://" . $_SERVER['SERVER_NAME'] . $_SERVER['PHP_SELF'];
                $ctrlconfi = "?controller=AdminModules&configure=" . Tools::getValue('controller');
                $urlLink = $hostlink . $ctrlconfi ."&token=" . $token;
                Tools::redirect($urlLink);
        }
    }
	
    /**
     * Initialization function called from Constructor.
	 * Redirect to module configure page when click on module tab
     */
    private function moduleSubpageUrl($subpage)
    {
		$token   = Tools::getAdminTokenLite('AdminModules');
		if(isset($_SERVER['REQUEST_SCHEME']) && $_SERVER['REQUEST_SCHEME']) $request_scheme = $_SERVER['REQUEST_SCHEME']; else $request_scheme = 'http';
		if(isset($_SERVER['REQUEST_SCHEME']) && $_SERVER['REQUEST_SCHEME']) $request_scheme = $_SERVER['REQUEST_SCHEME']; else $request_scheme = 'http';
		$hostlink = $request_scheme . "://" . $_SERVER['SERVER_NAME'] . $_SERVER['PHP_SELF'];
		$ctrlconfi = "?controller=AdminModules&configure=ranginesmspresta";
		$urlLink = $hostlink . $ctrlconfi ."&token=" . $token . "&subpage=" . $subpage;
		return $urlLink;
    }

    /**
     * Prepare the config page content.
     */
    public function getContent()
    {

        if (!extension_loaded('curl')) {
            return $this->displayError($this->l('cURL is not enabled. You should enable it before using thes module.'));
        }
        $output = null;

        $errors = array();
		include('include/submit.php');
        $auth = $this->panelAuth();
        $this->smarty->assign(array(
            'auth' => $auth,
            'alerttype' => 'info',
        ));
		if(isset($this->context->cookie->redirect_errors)){
			$output .= $this->displayError($this->context->cookie->redirect_errors);
			// delete old messages
			unset($this->context->cookie->redirect_errors);
		}
		if(isset($this->context->cookie->redirect_success_message)){
			$output .= '<div class="alert alert-success"><button type="button" class="close" data-dismiss="alert">×</button>'.$this->context->cookie->redirect_success_message.'<div></div></div>';
			// delete old messages
			unset($this->context->cookie->redirect_success_message);
		}
		$output .= $this->display(__FILE__, 'views/templates/admin/headerinfo.tpl');
        $gatewayDescription = $gatewayWarning = $gatewaySuccess = $gatewayError = null;
        if ($auth['ok'] == 'user') {
            $sendnumber = str_replace('+98', '', Configuration::get($this->prefix .'SMSNUMBER' , null , null , $this->shop_id));
            $sendKhadamatiNumber = str_replace('+98', '', Configuration::get($this->prefix .'SMSSENUMBER' , null , null , $this->shop_id));
            if (in_array($sendnumber, $auth['lines'])) {
                $gatewaySuccess = $this->l('The gateway configured correctly!').' '
                    .$this->l('Your remain credit is: ') .' '
                    .$auth['credit'].' '
                    .$this->l('rials').', '
                    .$this->l('And your account expire time is after').' '
                    .$auth['expireTime'].' '
                    .$this->l('days');
				if($auth['updateTime']) $gatewaySuccess .= ' ('.$this->l('Panel info check time:').' '.$this->timeago($auth['updateTime']).')';
				if($auth['expireTime'] < 30) $gatewaySuccess .= '<br>' . $this->l('Please contact to Rangine SMS Service to extend panel for one year.');
				if($auth['credit'] < 4000) $gatewaySuccess .= '<br>'.$this->l('Your credit is low.').'<a href="http://sms.rangine.ir" target="_blank">' . $this->l('Please go to your account and charge it.') . '</a>';
            } else {
                $gatewayWarning .= $this->l('The user and pass are correct but this line is not belong to you: ').$sendnumber;
            }
            if ($sendKhadamatiNumber != '' and
                $sendKhadamatiNumber != null and
                !in_array($sendKhadamatiNumber, $auth['lines'])) {
                $gatewayWarning .= $this->l(' Service line is not belong to you: ').$sendKhadamatiNumber;
            }
            if (Configuration::get($this->prefix .'NEWACADTETYPE' , null , null , $this->shop_id) == 'sample' or
                Configuration::get($this->prefix .'NEWACCUTETYPE' , null , null , $this->shop_id) == 'sample' or
                Configuration::get($this->prefix .'NEWORADTETYPE' , null , null , $this->shop_id) == 'sample' or
                Configuration::get($this->prefix .'NEWORCUTETYPE' , null , null , $this->shop_id) == 'sample' or
                Configuration::get($this->prefix .'UPORCUTETYPE' , null , null , $this->shop_id) == 'sample') {
                if ($sendKhadamatiNumber == '' or !in_array($sendKhadamatiNumber, $auth['lines'])) {
                    $gatewayError = $this->l('You select sample for one of your sms messages but you don\'t specify Service line or it is not belong to you: ').$sendKhadamatiNumber;
                }
            }
        } elseif ($auth['ok'] == 'demo') {
            $gatewayDescription = $this->l('You are testing our demo services. The maximum amount of utilization of these services depends on our support section. If you have questions about the system and purchase a panel, you can contact 09191483567 with a call, text message or telegram.');
        } else {
            $gatewayError = $auth['error'];
        }
        if (version_compare(_PS_VERSION_, '1.6.1', '>=') === false) {
            $gatewayDescription= $gatewayDescription.$gatewayWarning.$gatewaySuccess.$gatewayError;
        }



        $helper = new HelperForm();
		$helper->module = $this;
		$helper->name_controller = $this->name;
		$helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->show_toolbar = false;
		$helper->toolbar_scroll = false;
        $helper->table = $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG');
        $helper->allow_employee_form_lang = $allow_employee_form_lang ? $allow_employee_form_lang : 0;
        $this->fields_form = array();
        $helper->id = (int)Tools::getValue('id_carrier');
        $helper->identifier = $this->identifier;
        $helper->submit_action = $this->name;
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).
        '&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );
		if($auth['ok'] == 'user' or $auth['ok'] == 'demo') $disabled = 0; else $disabled = 1;
		if($submited != '') {
			$activePage = $submited;
		} else {
			$activePage = Tools::getValue('subpage') ? Tools::getValue('subpage') : 'welcome';
		}
		$output .= "<div class='row ranginesmsprestasettings'>";
        $output .= $this->moduleTabs($activePage);
		$output .= "<div class='col-lg-10 col-sm-9'>";
		switch($activePage){
			case 'gatewaysettings':
				include_once('include/gatewayform.php');
				$output .= $helper->generateForm(array($fields_form_gateway));
				break;
			case 'alertssettings':
				include_once('include/alertsform.php');
				$output .= $helper->generateForm(array($fields_form_alerts));
				break;
			case 'sendsms':
				if($disabled == 1) {
					$output .= '<div class="panel">	<div class="panel-heading"><i class="icon-paper-plane"></i>'.$this->l('Send SMS').'</div><div class="content">	<p>'.$this->l('Please config SMS Gateway first.').'</p></div></div>';
					break;
				}
				include_once('include/sendsmsform.php');
				$output .= $helper->generateForm(array($fields_form_send_sms));
				break;
			case 'sendtoall':
				if($disabled == 1) {
					$output .= '<div class="panel">	<div class="panel-heading"><i class="icon-paper-plane"></i>'.$this->l('Send SMS').'</div><div class="content">	<p>'.$this->l('Please config SMS Gateway first.').'</p></div></div>';
					break;
				}
				include_once('include/sendtoallform.php');
				$output .= $helper->generateForm(array($fields_form_send_all_customers));
				break;
			case 'smstexts':
				include_once('include/smstextform.php');
				$output .= $helper->generateForm(array($fields_form_sms_texts));
				break;
			case 'sentsms':
				$output .= $this->logsPage();
				break;
			case 'oosSubscribers':
				$output .= $this->oosSubscribersPage();
				break;
			case 'welcome':
			default:
				include_once('include/welcomepage.php');
				$output .= $welcomePage;
				break;			
		}
		$output .="</div>";
		$output .="</div>";
        return $output;
    }
	
    /**
     * Prepare Tabs for top of admin page
     */
	private function moduleTabs($activePage){
		$pagesArray = array(
		'welcome' => $this->l('Welcome'),
		'gatewaysettings' => $this->l('Gateway Settings'),
		'alertssettings' => $this->l('Alerts Settings'),
		'sendsms' => $this->l('Send SMS'),
		'sendtoall' => $this->l('Send to All'),
		'smstexts' => $this->l('SMS Texts'),
		'sentsms' => $this->l('Sent SMS'),
		'oosSubscribers' => $this->l('Out of Stock').$this->text['prosign'],
		);
		$output = "<div class='ranginesmsprestatabs col-lg-2 col-sm-3'><div class='list-group'>";
		foreach($pagesArray as $key => $value){
			if($activePage == $key) {
				$class = 'class="list-group-item active"';
				$href = '#';
			} else {
				$class = 'class="list-group-item"';
				$href = $this->moduleSubpageUrl($key);
			}
			
			$output .= "<a href='$href' $class>$value</a>";
		}
		$output .= '</div></div>';
		return $output;
	}

    public function getConfigFieldsValues()
    {
		include_once('include/configs.php');
		$output = array();
		foreach($this->configs as $key => $value){
			$output[$key] = Tools::getValue($key, Configuration::get($this->prefix . $key) , null , null , $this->shop_id);
		}
		$output['RECIVER'] = Tools::getValue('RECIVER');
		$output['SENDONETEXT'] = Tools::getValue('SENDONETEXT');
		$output['SENDALLTEXT'] = Tools::getValue('SENDALLTEXT');
		$output['SENTOALLSENDERLINE'] = '100009';
		$output['SENTOALLDELAYTIME'] = 15;
        return $output;
    }
	
	/**
	 * Add a css to all back office pages.
	 *
	 **/
    public function hookBackOfficeHeader()
    {
        $this->context->controller->addCSS($this->_path . 'views/css/icon.css');
		return '<script>var updateOrderStatusSmsAlert = '.Configuration::get($this->prefix .'UPDATEORDERC' , null , null , $this->shop_id).';var updateOrderTrackingSmsAlert = '.Configuration::get($this->prefix .'ORDERTRACKING' , null , null , $this->shop_id).';</script>';
    }
	
	/**
	 * Send SMS after Create new address
	 **/
    public function hookActionObjectAddressAddAfter($params)
    {
		if (Tools::getValue('controller') == 'AdminAddresses') {
			if(Tools::getValue('customer_add_address_alert') != 'on') return true;
		}
        if (!$this->enable) {
            return true;
        }
        if (Configuration::get($this->prefix . 'USERNAME' , null , null , $this->shop_id) == '' or
            Configuration::get($this->prefix . 'PASSWORD' , null , null , $this->shop_id) == '' or
            Configuration::get($this->prefix . 'NEWADDMOBILE' , null , null , $this->shop_id) == 0) {
            return true;
        } 
        if (is_object($params['object'])) {
            $newAddress = get_object_vars($params['object']);
			$vars = array(
				'{firstname}' => $newAddress['firstname'],
				'{lastname}' => $newAddress['lastname'],
				'{shop_name}' => $this->shop,
			);
			$phone = $newAddress['phone_mobile'];
			if ($phone and !Tools::strlen($phone) < 10) {
				if ($text = $this->generateSMStext($vars, 'newaddress', 'customer')) {
					$this->sendOne($text, $phone, $newAddress['id_customer'], $this->l('New Address'));
				}
			}
		}
        return true;
    } 
	
	/**
	 * Send SMS after Create new customer
	 **/
    public function hookActionCustomerAccountAdd($params)
    {
        if (!$this->enable) {
            return true;
        }
        if (Configuration::get($this->prefix . 'USERNAME' , null , null , $this->shop_id) == '' or
            Configuration::get($this->prefix . 'PASSWORD' , null , null , $this->shop_id) == '') {
            return true;
        }

        if (is_object($params['newCustomer'])) {
            $newCustomer = get_object_vars($params['newCustomer']);
        }
        $vars = array(
            '{firstname}' => $newCustomer['firstname'],
            '{lastname}' => $newCustomer['lastname'],
			'{shop_name}' => $this->shop,
        );

        if (Configuration::get($this->prefix . 'NEWCUSTOMERA' , null , null , $this->shop_id) != 0 and Configuration::get($this->prefix . 'ADMINPHONE' , null , null , $this->shop_id) !== '') {
            if ($text = $this->generateSMStext($vars, 'newacount', 'admin')) {
				$AdminPhone = Configuration::get($this->prefix . 'ADMINPHONE' , null , null , $this->shop_id);
				 $this->sendOne($text, $AdminPhone, '-', $this->l('New Customer'));
            }
        }
        if (Configuration::get($this->prefix.'NEWCUSTOMERC' , null , null , $this->shop_id) != 0) {
            $id_address      = Address::getFirstCustomerAddressId($newCustomer['id']);
            $customerAddress = new AddressCore($id_address);
            $phone           = $customerAddress->phone_mobile;

            if ($phone and Tools::strlen($phone) < 10) {
				if ($text = $this->generateSMStext($vars, 'newacount', 'customer')) {
					$this->sendOne($text, $phone, $newCustomer['id'], $this->l('New Customer'));
				}
			}
        }
        return true;
    }
	
	/**
	 * Send SMS after Create new order
	 **/
    public function hookActionValidateOrder($params)
    {
        if (!$this->enable) {
            return true;
        }
        if (Configuration::get($this->prefix . 'USERNAME' , null , null , $this->shop_id) == '' or
            Configuration::get($this->prefix . 'PASSWORD' , null , null , $this->shop_id) == '') {
            return true;
        }

        $order    = $params['order'];
        $customer = $params['customer'];
        $currency = $params['currency'];

        $vars = array(
            '{firstname}' => $customer->firstname,
            '{lastname}' => $customer->lastname,
            '{order_id}' => $order->id,
            '{order_reference}' => $order->reference,
            '{shop_name}' => $this->shop,
            '{payment}' => $order->payment,
            '{total_paid}' => $order->total_paid,
            '{currency}' => $currency->name,
        );
        if (Configuration::get($this->prefix . 'NEWORDERA' , null , null , $this->shop_id) != 0
            and Configuration::get($this->prefix . 'ADMINPHONE' , null , null , $this->shop_id) != '') {
			$adminSend = 1;
            if ($order->total_paid == 0 and Configuration::get($this->prefix . 'NEWORDERFA', null , null , $this->shop_id) == 0) {
                $adminSend = 0;
            }
			if ($adminSend and $text = $this->generateSMStext($vars, 'neworder', 'admin')) {
				$AdminPhone = Configuration::get($this->prefix . 'ADMINPHONE' , null , null , $this->shop_id);
				$this->sendOne($text, $AdminPhone, '-', $this->l('New Order'));
			}
        }
        if (Configuration::get($this->prefix . 'NEWORDERC' , null , null , $this->shop_id) != 0) {
			$customaerSend = 1;
            if ($order->total_paid == 0 and Configuration::get($this->prefix . 'NEWORDERFC' , null , null , $this->shop_id) == 0) {
                $customaerSend = 0;
            }
			if($customaerSend){
				$orderMobileNumberType = Configuration::get($this->prefix . 'ORDERMOBILE' , null , null , $this->shop_id);
				if(!$orderMobileNumberType or $orderMobileNumberType == 'orderaddress'){
					$address  = new Address(intval($order->id_address_invoice));
				}else{
					$id_address = Address::getFirstCustomerAddressId($customer->id);
					$address = new Address((int)($id_address));
				}
				if ($address->phone_mobile == null) {
					return true;
				}
				$phone = $address->phone_mobile;
				if ($phone and !Tools::strlen($phone) < 10 and $text = $this->generateSMStext($vars, 'neworder', 'customer')) {
					$this->sendOne($text, $phone, $customer->id, $this->l('New Order'));
				}
			}
        }
        return true;
    }
	
	/**
	 * Send SMS after update order status
	 **/
    public function hookActionOrderStatusPostUpdate($params)
    {
        $customer_alert = Tools::getValue('customer_update_order_state_alert');
        if (!$this->enable or $customer_alert != 'on') {
            return true;
        }
        if (Configuration::get($this->prefix . 'USERNAME' , null , null , $this->shop_id) == '' or
            Configuration::get($this->prefix . 'PASSWORD' , null , null , $this->shop_id) == '') {
            return true;
        }

        $order      = new Order((int) ($params['id_order']));
        $orderstate = $params['newOrderStatus'];
        $customer = new Customer((int) $order->id_customer);
		$orderMobileNumberType = Configuration::get($this->prefix . 'ORDERMOBILE' , null , null , $this->shop_id);
		if(!$orderMobileNumberType or $orderMobileNumberType == 'orderaddress'){
			$address  = new Address(intval($order->id_address_invoice));
		}else{
			$id_address = Address::getFirstCustomerAddressId($customer->id);
			$address = new Address((int)($id_address));
		}
        if ($address->phone_mobile == null) {
            return true;
        }
        $vars = array(
            '{firstname}' => $customer->firstname,
            '{lastname}' => $customer->lastname,
            '{order_id}' => $order->id,
            '{order_reference}' => $order->reference,
            '{shop_name}' => $this->shop,
            '{order_state}' => $orderstate->name,
        );

        if (!$text = $this->generateSMStext($vars, 'updateorder', 'customer')) {
            return;
        }

        $smsResult = $this->sendOne($text, $address->phone_mobile, $customer->id, $this->l('Update Order Status'));
        if ($smsResult == 'sent') {
            $this->adminDisplayInformation($this->l('The sms has sent.'));
        } else {
            $this->adminDisplayWarning($this->l('The sms can\'t be sent. Error: ') . $smsResult);
        }
        return true;
    }

	/**
	 * Add js && css of oos to front-end
	 **/
	public function hookDisplayHeader()
	{
		$this->page_name = Dispatcher::getInstance()->getController();
		if (in_array($this->page_name, array('product')) && $this->ml() == 'true')
		{
			$this->context->controller->addJS($this->_path.'views/js/product-sms-alert.js');
			$this->context->controller->addCSS($this->_path.'views/css/product-sms-alert.css', 'all');
		}
	}
	
	/**
	 * Add oos buttom on product page
	 **/
    public function hookActionProductOutOfStock($params)
    {
		if (!$this->enable or
			Configuration::get($this->prefix . 'USERNAME' , null , null , $this->shop_id) == '' or
            Configuration::get($this->prefix . 'PASSWORD' , null , null , $this->shop_id) == '' or
			!Configuration::get($this->prefix . 'UPDATEORDERC' , null , null , $this->shop_id) or
			Product::isAvailableWhenOutOfStock($params['product']->out_of_stock)) {
            return;
        }


		$context = Context::getContext();
		$id_product = (int)$params['product']->id;
		$id_product_attribute = 0;
		if($this->ml() == 'true') return $this->display(__FILE__, 'views/templates/front/oos-alert.tpl');
    }
	
/*     public function hookActionObjectProductUpdateBefore($params)
    {
		//error_log('params before update:'.urldecode(json_encode($params)));
        return true;
    }
	
    public function hookActionObjectProductUpdateAfter($params)
    {
		//error_log('params after update:'.urldecode(json_encode($params)));
        return true;
    } */

	/**
	 * Actions at update product quantity
	 **/	
    public function hookactionUpdateQuantity($params)
    {
		if (!$this->enable ) {
            return true;
        }
        if (Configuration::get($this->prefix . 'USERNAME' , null , null , $this->shop_id) == '' or
            Configuration::get($this->prefix . 'PASSWORD' , null , null , $this->shop_id) == '') {
            return true;
        }
        $id_product       = (int)$params['id_product'];
        $id_product_attribute = (int)$params['id_product_attribute'];
        $context = Context::getContext();
        $id_shop = (int)$context->shop->id;
        $id_lang = (int)$context->language->id;
        $product = new Product($id_product, false, $id_lang, $id_shop, $context);
        if (empty($product)) {
             return;
        }
        $product_name = Product::getProductName($id_product, $id_product_attribute, $id_lang);
        $product_has_attributes = $product->hasAttributes();
		
        $check_oos = ($product_has_attributes && $id_product_attribute)
                    || (!$product_has_attributes && !$id_product_attribute);
        $product_quantity = (int)$params['quantity'];

		$adminPhone = Configuration::get($this->prefix . 'ADMINPHONE' , null , null , $this->shop_id);
	// For out of stock notification to admin
		$isActivate = Configuration::get($this->prefix . 'OUTOFSTOCK' , null , null , $this->shop_id); 
		
		if ($isActivate != 0
		&& $adminPhone != ''
		&& $check_oos
		&& $product->active == 1
		&& $product_quantity <= Configuration::get($this->prefix . 'OUTOFSTOCKTHRESHOLD' , null , null , $this->shop_id)) {
			$vars = array(
				'{product_name}' =>  $product_name,
				'{product_id}' => $id_product,
				'{shop_name}' => $this->shop,
				'{product_quantity}' => $product_quantity,
			);
			if (!$text = $this->generateSMStext($vars, 'outofstock', 'admin')) {
				return;
			}
			$this->sendOne($text, $adminPhone, 'admin', $this->l('Out of Stock'));
		}
            
	// back in stock => send to subscribers
		$isActivate = Configuration::get($this->prefix . 'BACKTOSTOCK' , null , null , $this->shop_id); 

		if ($isActivate != 0
		&& $product_quantity > Configuration::get($this->prefix . 'OUTOFSTOCKTHRESHOLD' , null , null , $this->shop_id)
		) {
			if(Configuration::get($this->prefix . 'OOSAUTO' , null , null , $this->shop_id)){
				$vars = array(
					'{product_name}' =>  $product_name,
					'{product_id}' => $id_product,
					'{shop_name}' => $this->shop,
					'{product_url}' => $this->productShortLink($id_product),
				);
				if (!$text = $this->generateSMStext($vars, 'backtostock', 'customer')) {
					return;
				}

				$phoneNumbers = $this->getOOSNumbers($id_product , 	$id_product_attribute , $id_shop , $id_lang );
				$phones = array_column($phoneNumbers, 'phone_number');
				if(!count($phones)) return;
				$recivers = implode(';',$phones);
				$reciversCount = count($phones);
				$this->sendOne($text, $recivers, 'subscribers' , $this->l('Back to Stock'));
				if ($adminPhone) {
					//send report to admin
					$adminText = $this->l('This massage has sent to Subscribers:') . '('.$reciversCount . ' ' . $this->l('Numbers').')~'.$text;
					$this->sendOne($adminText, $adminPhone, 'admin', $this->l('Back to Stock Report'));
				}
				$this->clearOOS(null, $id_product, $id_product_attribute, $id_shop, $id_lang);
			}elseif ($adminPhone) {
				//send report to admin
				$phoneNumbers = $this->getOOSNumbers($id_product , 	$id_product_attribute , $id_shop , $id_lang );
				$phones = array_column($phoneNumbers, 'phone_number');
				$reciversCount = count($phones);
				if(!$reciversCount) return;
				$adminText = $this->l('Hi Admin') . '~'. $reciversCount . ' ' . $this->l('users has subscribed to this product. Please go to your shop admin panel to send them a massage manualy');
				$this->sendOne($adminText, $adminPhone, 'admin', $this->l('Back to Stock Report'));
			}
		}
		return true;
			
	}
	
	/**
	 * Send sms on set order tracking number
	 **/	
	 public function hookactionAdminOrdersTrackingNumberUpdate($params)
    {
        $customer_alert = Tools::getValue('customer_update_order_tracking_alert');
        if (!$this->enable or $customer_alert != 'on') {
            return true;
        }
        $order = $params['order'];
        $customer = $params['customer'];
        $carrier = $params['carrier'];
        $carrier = $params['carrier'];
		$orderMobileNumberType = Configuration::get($this->prefix . 'ORDERMOBILE' , null , null , $this->shop_id);
		if(!$orderMobileNumberType or $orderMobileNumberType == 'orderaddress'){
			$address  = new Address(intval($order->id_address_invoice));
		}else{
			$id_address = Address::getFirstCustomerAddressId($customer->id);
			$address = new Address((int)($id_address));
		}

        if ($address->phone_mobile == null) {
            return true;
        }	

        $vars = array(
            '{firstname}' => ($customer->firstname),
            '{lastname}' => ($customer->lastname),
            '{order_id}' => $order->id,
            '{shop_name}' => $this->shop,
            '{carrier}' => $carrier->name,
            '{tracking}' => $order->shipping_number,
        );

        if (!$text = $this->generateSMStext($vars, 'updateOrderTracking', 'customer')) {
            return;
        }
        $smsResult = $this->sendOne($text, $address->phone_mobile, $customer->id, $this->l('Update Tracking Number'));
        if ($smsResult == 'sent') {
            $this->adminDisplayInformation($this->l('The sms has sent.'));
        } else {
            $this->adminDisplayWarning($this->l('The sms can\'t be sent. Error: ') . $smsResult);
        }
        return true;
    }
	
	/**
	 * Make pattern SMS humanity
	 **/
	private function patternSmsHumanity($text){
		$splited = explode(';', $text);
		$pattern_code_array = explode(':', $splited[0]);
		$pattern_code = $pattern_code_array[1];
		unset($splited[0]);
		$resArray = array();
		foreach ($splited as $parm) {
			$splited_parm = explode(':', $parm);
			$resArray[$splited_parm[0]] = $splited_parm[1];
		}
		switch($pattern_code){
			case '142':
				$result = $resArray['firstname'].' '. $resArray['lastname'].' در سايت '. $resArray['site'].' ثبت نام نمود.';
				break;
			case '144':
				$result = "سفارش جديد: {$resArray['order-code']}~مشتري: {$resArray['customer']}~جمع خريد: {$resArray['amount']} {$resArray['currency']}";
				break;
			case '143':
				$result = "{$resArray['firstname']} {$resArray['lastname']} عزيز،~از اینکه عضو {$resArray['system-name']} شدید سپاسگزاریم.";
				break;
			case '145':
				$result = "مشتری گرامی {$resArray['name']}،~سفارش شماره {$resArray['order-code']} به مبلغ {$resArray['amount']} ايجاد گرديد.~{$resArray['company']}";
				break;
			case '152':
				$result = "مشتري گرامي {$resArray['name']}،~سفارش {$resArray['order']} در وضعيت {$resArray['status']} قرار گرفت.~{$resArray['company']}";
				break;
			case '165':
				$result = "مشتری گرامی {$resArray['name']}،~سفارش {$resArray['order']} با {$resArray['service']} و شماره پیگیری زیر ارسال شده است:~{$resArray['tracking-code']}~{$resArray['company']}";
				break;
			default:
				$result = $text;
				break;
		}
		return $result;
	}
    /** generate text message
     * @param array vars varible to set in tamplate
     * @param string event
     * @param string reciver
     * @return text message
     */
    private function generateSMStext($vars, $event, $reciver)
    {
        $serviceLineStatus = true;
        $sendServiceNumber = str_replace('+98', '', Configuration::get($this->prefix .'SMSSENUMBER' , null , null , $this->shop_id));
        if ($sendServiceNumber == '') {
            $serviceLineStatus = false;
        }
        if ($reciver == 'admin') {
            switch ($event) {
                case 'newacount':
                    $type = Configuration::get($this->prefix .'NEWACADTETYPE' , null , null , $this->shop_id);
                    if ($type == 'custom') {
                        $massage = Configuration::get($this->prefix .'NEWACADTEXT' , null , null , $this->shop_id);
                    } elseif ($type == 'sample' and $serviceLineStatus) {
                        $massage = "patterncode:142;firstname:{firstname};lastname:{lastname};site:".$this->shop;
                    } else {
                        $massage = $this->l('{firstname} {lastname} registred on site ').$this->shop;
                    }
                    break;
                case 'neworder':
                    $type = Configuration::get($this->prefix .'NEWORADTETYPE' , null , null , $this->shop_id);
                    if ($type == 'custom') {
                        $massage = Configuration::get($this->prefix .'NEWORADTEXT' , null , null , $this->shop_id);
                    } elseif ($type == 'sample' and $serviceLineStatus) {
                        $massage = "patterncode:144;customer:{firstname} {lastname};";
                        $massage .= "order-code:{order_id};amount:{total_paid};currency:{currency}";
                    } else {
                        $massage = $this->l('New order at {shop_name}~Customer{firstname} {lastname}~Order: {order_id}~Payment: {payment}~Total: {total_paid} {currency}');
                    }
                    break;
                case 'outofstock':
                    $type = Configuration::get($this->prefix .'OUTOFSTOCKTETYPE' , null , null , $this->shop_id);
                    if ($type == 'custom') {
                        $massage = Configuration::get($this->prefix .'OUTOFSTOCKTEXT' , null , null , $this->shop_id);
                    } elseif ($type == 'sample' and $serviceLineStatus) { // باید ایجاد و جایگزین شود
                        $massage = "patterncode:291;product_id:{product_id};";
                        $massage .= "product_name:{product_name};shop_name:{shop_name}";
                    } else {
                        $massage = $this->l('The product by id: {product_id} {product_name} goes out of stock.~{shop_name}');
                    }
                    break;
            }
        } else {
            switch ($event) {
                case 'newacount':
                    $type = Configuration::get($this->prefix .'NEWACCUTETYPE' , null , null , $this->shop_id);
                    if ($type == 'custom') {
                        $massage = Configuration::get($this->prefix .'NEWACCUTEXT' , null , null , $this->shop_id);
                    } elseif ($type == 'sample' and $serviceLineStatus) {
                        $massage = "patterncode:143;firstname:{firstname};lastname:{lastname};system-name:".$this->shop;
                    } else {
                        $massage = $this->l('Dear {firstname} {lastname},~Welcome to our site.');
                    }
                    break;
                case 'neworder':
                    $type = Configuration::get($this->prefix .'NEWORCUTETYPE' , null , null , $this->shop_id);
                    if (Configuration::get($this->prefix .'INVOICE_NEWORDER' , null , null , $this->shop_id) == 1) {
                        $invoceLink = $this->invoiceShortLink($vars['{order_id}']);
                        $invoiceText = $this->l('~Invoice:') . ' ' . str_replace('https://', '', $invoceLink);
                    } else {
                        $invoiceText = '';
                    }
                    if ($type == 'custom') {
                        $massage = Configuration::get($this->prefix .'NEWORCUTEXT' , null , null , $this->shop_id) . $invoiceText;
                    } elseif ($type == 'sample' and $serviceLineStatus) {
                        if ($invoiceText != '') {
                            $vars['{shop_name}'] = $vars['{shop_name}'] . "\nصورتحساب  " . str_replace('https://', '', $invoceLink);
                        }
                        $massage = "patterncode:145;name:{firstname} {lastname};order-code:{order_id};amount:{total_paid} {currency};company:{shop_name}";
                    } else {
                        $massage = $this->text['NEWORCUTEXT'] . $invoiceText;
                    }
                    break;
                case 'updateorder':
                    $type = Configuration::get($this->prefix .'UPORCUTETYPE' , null , null , $this->shop_id);
                    if (Configuration::get($this->prefix .'INVOICE_UPDATEORDER' , null , null , $this->shop_id) == 1) {
                        $invoceLink = $this->invoiceShortLink($vars['{order_id}']);
                        $invoiceText = $this->l('~Invoice:') . ' ' . str_replace('https://', '', $invoceLink);
                    } else {
                        $invoiceText = '';
                    }
                    if ($type == 'custom') {
                        $massage = Configuration::get($this->prefix .'UPORCUTEXT' , null , null , $this->shop_id) . $invoiceText;
                    } elseif ($type == 'sample' and $serviceLineStatus) {
                        if ($invoiceText != '') {
                            $vars['{shop_name}'] = $vars['{shop_name}'] . "\nصورتحساب  " . str_replace('https://', '', $invoceLink);
                        }
                        $massage = "patterncode:152;name:{firstname} {lastname};order:{order_id};";
                        $massage .= "status:{order_state};company:{shop_name}";
                    } else {
                        $massage = $this->text['UPORCUTEXT'] . $invoiceText;
                    }
                    break;
                case 'updateOrderTracking':
                    $type = Configuration::get($this->prefix .'UPORTRTETYPE' , null , null , $this->shop_id);
                    if (Configuration::get($this->prefix .'INVOICE_UPDATETRACKING' , null , null , $this->shop_id) == 1) {
                        $invoceLink = $this->invoiceShortLink($vars['{order_id}']);
                        $invoiceText = $this->l('~Invoice:') . ' ' . str_replace('https://', '', $invoceLink);
                    } else {
                        $invoiceText = '';
                    }
                    if ($type == 'custom') {
                        $massage = Configuration::get($this->prefix .'UPORTRTEXT' , null , null , $this->shop_id) . $invoiceText;
                    } elseif ($type == 'sample' and $serviceLineStatus) {
                        if ($invoiceText != '') {
                            $vars['{shop_name}'] = $vars['{shop_name}'] . "\nصورتحساب  " . str_replace('https://', '', $invoceLink);
                        }
                        $massage = "patterncode:165;name:{firstname} {lastname};order:{order_id};";
                        $massage .= "service:{carrier};company:{shop_name};";
                        $massage .= "tracking-code:{tracking}";
                    } else {
                        $massage = $this->text['UPORTRTEXT'] . $invoiceText;
                    }
                    break;
                case 'newaddress':
                    $type = Configuration::get($this->prefix .'NEWADDRTETYPE' , null , null , $this->shop_id);
                    if ($type == 'custom') {
                        $massage = Configuration::get($this->prefix .'NEWADDRTEXT' , null , null , $this->shop_id);
                    } elseif ($type == 'sample' and $serviceLineStatus) {
                        $massage = "patterncode:143;firstname:{firstname};lastname:{lastname};system-name:".$this->shop;
                    } else {
                        $massage = $this->l('Dear {firstname} {lastname},~Welcome to our site.');
                    }
                    break;
                case 'backtostock':
                    $type = Configuration::get($this->prefix .'BACKTOSTOCKTETYPE' , null , null , $this->shop_id);
                    if ($type == 'custom') {
                        $massage = Configuration::get($this->prefix .'BACKTOSTOCKTEXT' , null , null , $this->shop_id);
                    } elseif ($type == 'sample' and $serviceLineStatus) { // باید ایجاد و جایگزین شود
                        $massage = $this->l('{product_name} goes back to stock!~{product_url}~{shop_name}');
                    } else {
                        $massage = $this->l('{product_name} goes back to stock!~{product_url}~{shop_name}');
                    }
                    break;
            }
        }

        $template = str_replace(array_keys($vars), array_values($vars), $massage);
        return $template;
    }

    /** save logs to database
     * @param string status
     * @param string logs form webservice
     * @return boolean
     */
    public function saveLogs($status, $description, $bulk = null, $phone = null, $customer = null, $position = null,$type)
    {	
		$time = time();
		$context = Context::getContext();
        $id_shop = (int)$context->shop->id;
        $fields = array(
            'customer' => pSQL($customer),
            'phone' => pSQL($phone),
            'position' => pSQL($position),
            'status' => pSQL($status),
            'bulk' => pSQL($bulk),
            'description' => pSQL($description),
            'timestamp' => pSQL($time),
            'type' => pSQL($type),
            'shop' => pSQL($id_shop),
        );
		if($this->demo()) $fields['delivery'] = 'demo';
		if($type == 'bysample') $fields['delivery'] = 'bysample';
        return Db::getInstance()->insert($this->name, $fields);
    }

    /** Get logs from database
     * @param string status
     * @param string logs form webservice
     * @return boolean
     */
    public function getLogs($page = 0, $onpage = 50, $bulk = null, $phone = null, $customer = null, $position = null, $shop_id = null)
    {
		$offset = $page * $onpage;
		$sql = new DbQuery();
		$sql->select('*');
		$sql->from($this->name, 't');
		if(!is_null($bulk))$sql->where("t.bulk = '{$bulk}'");
		if(!is_null($phone))$sql->where("t.phone = '{$phone}'");
		if(!is_null($customer))$sql->where("t.customer = '{$customer}'");
		if(!is_null($position))$sql->where("t.position = '{$position}'");
		if(!is_null($shop_id))$sql->where("t.shop = '{$shop_id}'");
		$sql->orderBy('id_ranginesmspresta DESC');
		$sql->limit($onpage, $offset = 0);
		return Db::getInstance()->executeS($sql);
    }
    /** Get logs from database
     * @param string status
     * @param string logs form webservice
     * @return boolean
     */
    public function updateLogs($bulk,$status)
    {
		Db::getInstance()->update($this->name, array('delivery' => pSQL($status)), 'bulk = ' . $bulk);
    }
    /** Create sent logs page
     * @param string status
     * @param string logs form webservice
     * @return boolean
     */
    public function logsPage($page = 0, $onpage = 20)
    {
		include_once('include/sentsms.php');
		return $output;
    }
    /** create Out of Stock pruducts subescribers page
     * @return table
     */
    public function oosSubscribersPage($page = 0, $onpage = 20)
    {
		include_once('include/oos.php');
		return $output;
    }

	/** Get oos from database
     * @param string status
     * @param string logs form webservice
     * @return boolean
     */
    public function getOOS($page = 0, $onpage = 50, $customer_id = null, $phoneNumber = null, $id_product = null, $id_product_attribute = null, $shop_id = null, $id_lang = null)
    {
		$offset = $page * $onpage;
		$sql = new DbQuery();
		$sql->select('*');
		$sql->select('COUNT(phone_number) as phones');
		$sql->from('ranginesmspresta_oos', 't');
		if(!is_null($customer_id))$sql->where("t.id_customer = '{$customer_id}'");
		if(!is_null($phoneNumber))$sql->where("t.phone_number = '{$phoneNumber}'");
		if(!is_null($id_product))$sql->where("t.id_product = '{$id_product}'");
		if(!is_null($id_product_attribute))$sql->where("t.id_product_attribute = '{$id_product_attribute}'");
		if(!is_null($shop_id))$sql->where("t.id_shop = '{$shop_id}'");
		if(!is_null($id_lang))$sql->where("t.id_lang = '{$id_lang}'");
		$sql->orderBy('id_product');
		$sql->groupBy('id_product_attribute');
		//$sql->limit($onpage, $offset);
		return Db::getInstance()->executeS($sql);
	}
	
	/** Get oos phoneNumbers from database
     * @param string status
     * @param string logs form webservice
     * @return boolean
     */
	private function getOOSNumbers($id_product = null, 	$id_product_attribute = null, $shop_id = null, $id_lang = null)
		{
			$sql = new DbQuery();
			$sql->select('t.phone_number');
			$sql->from('ranginesmspresta_oos', 't');
			if(!is_null($id_product))$sql->where("t.id_product = '{$id_product}'");
			if(!is_null($id_product_attribute))$sql->where("t.id_product_attribute = '{$id_product_attribute}'");
			if(!is_null($shop_id))$sql->where("t.id_shop = '{$shop_id}'");
			if(!is_null($id_lang))$sql->where("t.id_lang = '{$id_lang}'");
			return Db::getInstance()->executeS($sql);
	}	
	/** Clear oos from database
     * @param string status
     * @param string logs form webservice
     * @return boolean
     */
    public function clearOOS($phoneNumber = null, $id_product = null, $id_product_attribute = null, $shop_id = null, $id_lang = null)
    {
		$whereArray = array();
		if(!is_null($phoneNumber) && $phoneNumber !== '') $whereArray[] = "phone_number = '{$phoneNumber}'";
		if(!is_null($id_product)) $whereArray[] = "id_product = '{$id_product}'";
		if(!is_null($id_product_attribute)) $whereArray[] = "id_product_attribute = '{$id_product_attribute}'";
		if(!is_null($shop_id)) $whereArray[] = "id_shop = '{$shop_id}'";
		if(!is_null($id_lang)) $whereArray[] = "id_lang = '{$id_lang}'";
		if(count($whereArray)) $where = implode(' AND ',$whereArray); else $where = '';
		//return $where;
		return Db::getInstance()->delete('ranginesmspresta_oos', $where);
	}
	/**
	 * Show delivery status in logs page table
	 *
	 */
	protected function deliveryStatus($logdelivery = null ,$bulk,$type){
		if($type == 'bysample') return $this->l('by sample');
		switch($logdelivery){
			case 'demo':
				return $this->l('demo');
				break;
			case 'bysample':
				return $this->l('by sample');
				break;	
			case 'notsync':
			case 'send':
			case 'pending':
			case 'failed':
			case 'discarded':
			case 'delivered':
				return $this->deliveryStatusTranslate($logdelivery);
				break;
			default:
				if(!is_null($bulk)) {
					return '<span class="checkdelivery"><span class="bulk">'.$bulk.'</span><span class="loader"></span></span>';
				} else {
					return '';
				}
				break;
		}
	}
	
	/**
	 * translate machine name of status to human lang
	 *
	 */
    public function deliveryStatusTranslate($status){
		$responseArray = array(
			'notsync' =>     $this->l('Not Sync'),
			'send' =>         $this->l('sent'),
			'pending' =>     $this->l('pending'),
			'failed' =>     $this->l('failed'),
			'discarded' =>     $this->l('discarded'),
			'delivered' =>     $this->l('delivered'),
		);
		if(isset($responseArray[$status])) return $responseArray[$status]; else return $this->l('Not Defined:') . $status;
	}
	
	/** get phone mobile from customers
     * @return strings of phonemobiles
     */
    protected function getPhoneMobiles($getTotal = false)
    {
        //get mobiles
        $mobilesArray = array();
        $mobiles = null;
        if (!$getTotal) {
            $sql = new DbQueryCore();
            $sql->select('DISTINCT phone_mobile');
            $sql->from('address');
            $query = $sql->build();

            $results=Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($query);
            if ($results and count($results)>0) {
                foreach ($results as $result) {
                    if (!Tools::isEmpty($result['phone_mobile'])) {
                        $mobilesArray[]=$result['phone_mobile'];
                    }
                }
                $mobiles = implode(';', $mobilesArray);
            } else {
                return false;
            }
            return $mobiles;
        } else {
            $sql= new DbQueryCore();
            $sql->select('count(DISTINCT phone_mobile)');
            $sql->from('address');
            $sql->where('phone_mobile !=""');
            $query= $sql->build();
            return Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);
        }
    }

    /** get SMS panel expire time
    * @return a number remain days
    */
    private function panelExpireTime()
    {
        $param = array('op'=>'usertime');
        $connectpanel = $this -> connectWebservice($param);
        if ($connectpanel) {
            $time = strtotime($connectpanel);
            $now = new DateTime;
            $expire = new DateTime('@'.$time);
            $diff = $now->diff($expire);
            $remaindays = $diff->days;
        } else {
            $remaindays = 'روز';
        }
        return $remaindays;
    }
	
    private function ml()
    {
		$l = $this->license;
		if($l == '') return 'null';
		$lCheckTime = Configuration::get($this->prefix . 'LICENSETIME');
		if($lCheckTime == '') return 'retry';
		$lastWeekTime = time() - 604800;
		if($lCheckTime < $lastWeekTime) return 'retry';
		return 'true';		
	}
    /** get SMS panel Authentication
    * @return array of OK status, panel lines, expire time, remain credits
    */
    private function panelAuth()
    {
        if (Configuration::get($this->prefix . 'USERNAME' , null , null , null) == 'demo' &&
            Configuration::get($this->prefix . 'PASSWORD' , null , null , null) == 'demo') {
            return array(
                'ok' => 'demo',
                'credit' => 0,
                'expireTime' => 0,
                'lines' => 0,
            );
        }
		if(Configuration::get($this->prefix . 'PANELAUTH' , null , null , $this->shop_id)){
			$authCache = unserialize(Configuration::get($this->prefix . 'PANELAUTH' , null , null , $this->shop_id));
			if($authCache['updateTime'] > time()-86400) return $authCache;		
		}
		$param = array('op'=>'credit');
		$connectpanel = $this -> connectWebservice($param);
		if (isset($connectpanel['status']) and $connectpanel['status'] == 'failed') {
			return array('ok' => false,'error' => $connectpanel['res_data']);
		}
		if ($connectpanel[0] == 0) {
			$authCache = array(
				'ok' => 'user',
				'credit' => round($connectpanel[1]),
				'expireTime' => $this -> panelExpireTime(),
				'lines' => $this -> panelLines(),
				'updateTime' => time(),
			);

			Configuration::updateValue($this->prefix . 'PANELAUTH' , serialize($authCache));
			return $authCache;
		} else {
			return array(
				'ok' => 'failed',
				'error' => $connectpanel[1],
			);
		}
		
    }

    /** get SMS panel available user lines
    * @return array of SMS lines numbers
    */
    private function panelLines()
    {
        $param = array('op'=>'lines');
        $connectpanel = $this -> connectWebservice($param);
        return $connectpanel;
    }

    /** get SMS panel remain credit
    * @return a number
    */
    private function panelCredit()
    {
        $param = array('op'=>'credit');
        $connectpanel = $this -> connectWebservice($param);
        return round($connectpanel[1]);
    }

    /** get SMS panel important news
    * @return string to show at top of config page
    */
    private function panelNews()
    {
		$auth = $this->panelAuth();
        $url = $this->support.'/smsapi/prestashop/'.$this->name.'/?getNews';
		if($auth['ok'] !== 'user') $url .= '&registry';
		$l = $this->ml();
		if($l == 'null'){
			$url .= '&l=form';
		}elseif($l == 'retry'){
			$url .= '&l=retry';
		}
		$url .= '&lic=' . $this->license;
        $param = array(
            'Mversion' => $this->version,
            'PSversion' => $this->PSversion,
            'shop' => $this->shop,
            'domain' => $this->domain,
            );
        $handler = curl_init($url);
		curl_setopt($handler, CURLOPT_CONNECTTIMEOUT, 10); 
		curl_setopt($handler, CURLOPT_TIMEOUT, 30);
        curl_setopt($handler, CURLOPT_CUSTOMREQUEST, "POST");
        curl_setopt($handler, CURLOPT_POSTFIELDS, $param);
        curl_setopt($handler, CURLOPT_RETURNTRANSFER, true);
        $response = curl_exec($handler);

        return $response;
    }
	

    /** Create a short link for long invoice link by goo.gl api
    * @parm $orderID number
    * @return short link or null if there is a problem
    */
    private function invoiceShortLink($orderID = null)
    {
        if ($orderID == null) {
            return '';
        }
        $factorUrl = $this->domain . '/index.php?controller=pdf-invoice&id_order=' . $orderID;
        $param = array(
            'longUrl' => $factorUrl,
            );
        $url = $this->support.'/smsapi/prestashop/'.$this->name.'/?urlShortener';
        $handler = curl_init($url);
        curl_setopt($handler, CURLOPT_CUSTOMREQUEST, "POST");
        curl_setopt($handler, CURLOPT_POSTFIELDS, $param);
        curl_setopt($handler, CURLOPT_RETURNTRANSFER, true);
        $response = curl_exec($handler);
        if ($response == 'false' or $response == '') {
            return '';
        } else {
            return $response;
        }
    }

    /** Create a short link for long product link by goo.gl api
    * @parm $orderID number
    * @return short link or null if there is a problem
    */
    private function productShortLink($productID = null)
    {
        if ($productID == null) {
            return '';
        }
		
        $productUrl = $this->domain . '/index.php?controller=product&id_product=' . $productID;
        $param = array(
            'longUrl' => $productUrl,
            );
        $url = $this->support.'/smsapi/prestashop/'.$this->name.'/?urlShortener';
        $handler = curl_init($url);
        curl_setopt($handler, CURLOPT_CUSTOMREQUEST, "POST");
        curl_setopt($handler, CURLOPT_POSTFIELDS, $param);
        curl_setopt($handler, CURLOPT_RETURNTRANSFER, true);
        $response = curl_exec($handler);
        if ($response == 'false' or $response == '') {
            return '';
        } else {
            return $response;
        }
    }

    /** Get one sent sms delivery
    * @return a string of sms status
    */
    public function panelGetDelivery($uinqid)
    {
		$param = array('op'=>'delivery','uinqid'=>$uinqid);
		$connectpanel = $this -> connectWebservice($param);
		return $connectpanel;
    }

    /** Get if user set deo as username and password
    * @return boolean
    */
    private function demo()
    {
        $user = Configuration::get($this->prefix . 'USERNAME' , null , null , $this->shop_id);
        $pass = Configuration::get($this->prefix . 'PASSWORD' , null , null , $this->shop_id);	
        if ($user == 'demo' and $pass == 'demo') {
            return true;
        } else {
            return false;
        }
    }
	
	private function timeago($timestamp) {

	   
	   $strTime = array(
					$this->l('second'),
					$this->l('minute'),
					$this->l('hour'),
					$this->l('day'),
					$this->l('month'),
					$this->l('year')
					);
	   $length = array("60","60","24","30","12","10");

	   $currentTime = time();
	   if($currentTime >= $timestamp) {
			$diff     = time()- $timestamp;
			for($i = 0; $diff >= $length[$i] && $i < count($length)-1; $i++) {
			$diff = $diff / $length[$i];
			}

			$diff = round($diff);
			return $diff . ' ' . $strTime[$i] . ' ' . $this->l('(s) ago ');
	   }
	}
    /** prepare one sms to send
    * @param string $text
    * @param string $phonenumber
    * @param number $customer : customer ID
    * @param string $position : where sms message sent
    * @return an array
    */
    protected function sendOne($text, $phonenumber, $customer = null, $position = null)
    {
        //If text is sample
        if (Tools::substr($text, 0, 11) === "patterncode") {
            $splited = explode(';', $text);
            $pattern_code = explode(':', $splited[0])[1];
            unset($splited[0]);
            $resArray = array();
            foreach ($splited as $parm) {
                $splited_parm = explode(':', $parm);
                $resArray[$splited_parm[0]] = $splited_parm[1];
            }
            if ($this->demo()) {
                $param = array(
                        'uname' => 'demo',
                        'pass' => 'demo',
                        'message' => $text,
                        'to' => $phonenumber,
                        'op' => 'sendPattern',
                    );
                $res = $this -> connectWebservice($param);
                if ($res['status'] == 'sent') {
                    $status = $res['status'];
                    $description = 'pattern:' . $text;
                    $bulk = $res['result'];
                    $this->saveLogs($status, $description, $bulk, $phonenumber, $customer, $position,'bysample');
                    return 'sent';
                } else {
                    $status = 'failed';
                    $description = 'Text:' . $text . ' - error: ' . $res['result'];
                    $bulk = '-';
                    $this->saveLogs($status, $description, $bulk, $phonenumber, $customer, $position,'bysample');
                    return $res['result'];
                }
            }
            $user = Configuration::get($this->prefix . 'USERNAME' , null , null , $this->shop_id);
            $pass = Configuration::get($this->prefix . 'PASSWORD' , null , null , $this->shop_id);
            $fromNum = Configuration::get($this->prefix . 'SMSSENUMBER' , null , null , $this->shop_id);
            $toNum = explode(';', $phonenumber);
            $input_data = $resArray;
			$url = "http://sms.rangine.ir/patterns/pattern?username=".$user."&password=".urlencode($pass)."&from=".$fromNum."&to=".json_encode($toNum)."&input_data=".urlencode(json_encode($input_data))."&pattern_code=".$pattern_code;
			$param = array();
			$handler = curl_init($url);             
			curl_setopt($handler, CURLOPT_CUSTOMREQUEST, "POST");
			curl_setopt($handler, CURLOPT_POSTFIELDS, $param);                       
			curl_setopt($handler, CURLOPT_RETURNTRANSFER, true);
			$response = curl_exec($handler);
            $response2 = json_decode($response);
            if (is_array($response2)) {
                $res_code = $response2[0];
                $res_data = $response2[1];
                $errorCodes = array(
                    '962' => $this->l('The username or password is incorrect!'),
                    '971' => $this->l('Patterns is not available!'),
                    '970' => $this->l('Parameters are incorrect!'),
                    '972' => $this->l('The reciver number is not correct!'),
                );
                $status = 'failed';
                if (isset($errorCodes[$res_code])) {
                    $result = $errorCodes[$res_code];
                } else {
                    $result = $res_data.' - Error Code:'.$res_code;
                }
                $this->saveLogs($status, $text, '-', $phonenumber, $customer, $position,'bysample');
            } else {
                $status = $result = 'sent';
                $this->saveLogs($status, $text, $response, $phonenumber, $customer, $position,'bysample');
            }
            return $result;
        }
        $rcpt_nm = explode(';', $phonenumber);
        $param = array();
        $param = array
                    (
                        'message'=> $text,
                        'to'=>json_encode($rcpt_nm),
                        'op'=> 'send'
                    );
		if($position == $this->l('Send Customers'))	{
			$sender = Tools::getValue('SENTOALLSENDERLINE');
			$delay = Tools::getValue('SENTOALLDELAYTIME');
			$param['from'] = Tools::getValue('SENTOALLSENDERLINE');
			$param['time'] = date("Y-m-d H:i:s", time()+($delay*60));
		}elseif (count($rcpt_nm) > 5) {
            $param['time'] = date("Y-m-d H:i:s", time()+300);
            $phonenumber = 'multiple';
        }
        $connectpanel = $this -> connectWebservice($param);
        $status = $connectpanel['status'];
        $description = 'Text:' . $text;
        $bulk = $connectpanel['res_data'];
        $this->saveLogs($status, $description, $bulk, $phonenumber, $customer, $position,'normal');
        return $connectpanel['result'];
    }

    /** public function to use by other modules
    * @param string $param['message']: sms massage
    * @param string $param['reciver']: recivers delimated by ;
    * @return boolean
    */
    public function sendSMS($param = array() , $position = null)
    {
        // $rcpt_nm = explode(';', $param['reciver']);
        // $param['to'] = json_encode($rcpt_nm);
        // $param['op'] = 'send';
		$text = $param['message'];
		$phonenumber = $param['reciver'];
        $result = $this -> sendOne($text, $phonenumber, $customer = null, $position);
        if ($result == 'sent') {
            return true;
        } else {
            return false;
        }
    }

    /** connect to webservice
    * @param array multiple as indexed in webservice api document
    * @return different result on different options
    */
    protected function connectWebservice($param = array())
    {
        $url = "sms.rangine.ir/services.jspd";
        $param['uname'] = Configuration::get($this->prefix . 'USERNAME' , null , null , null);
        $param['pass'] = Configuration::get($this->prefix . 'PASSWORD' , null , null , null);
		if(!isset($param['from']) || $param['from'] == '') $param['from']= Configuration::get($this->prefix . 'SMSNUMBER' , null , null , null);

        if (isset($param['message'])) {
            $param['message'] = str_replace('~', "\n", $param['message']);
        }
        if ($this->demo()) {
            $url = $this->support.'/smsapi/prestashop/'.$this->name.'/?demo';
        } elseif (!$param['uname'] or !$param['pass'] or !$param['from']) {
            return array(
                'status' => 'failed',
                'result' => $this->l('Gateway configuration is not set proprely.'),
                'res_data' => $this->l('Gateway configuration is not set proprely.')
                );
        }
        $handler = curl_init($url);
        curl_setopt($handler, CURLOPT_CUSTOMREQUEST, "POST");
        curl_setopt($handler, CURLOPT_POSTFIELDS, $param);
        curl_setopt($handler, CURLOPT_RETURNTRANSFER, true);
        $response = curl_exec($handler);
        $response2 = json_decode($response);
        $res_code = $response2[0];
        $res_data = $response2[1];
        switch ($param['op']) {
            case 'sendPattern':
                $result = array('status' => $res_code,'result' => $res_data);
                break;
            case 'usertime':
                $result = $res_data;
                break;
            case 'credit':
                $result = $response2;
                break;
            case 'delivery':
                if ($response == 'null') {
                    return null;
                }
                if ($res_code == 0) {
                    $result = json_decode($res_data);
                    $res2 = explode(':', $result[0]);
                    $res_data = $res2[1];
                }
                $result = $res_data;
                break;
            case 'lines':
                $linesArray = json_decode($res_data);
                $lines = array();
                foreach ($linesArray as $line) {
                    $lines[] = str_replace('+98', '', json_decode($line)->number);
                }
                $result = $lines;
                break;
            case 'send':
                $errorCodes = array(
                    '1' =>    $this->l('The message is empty!'),
                    '2' =>    $this->l('The user is limited!'),
                    '3' =>    $this->l('The Number is not belong to you!'),
                    '4' =>    $this->l('You have no reciver!'),
                    '5' =>    $this->l('You do not have enough credit!'),
                    '6' =>    $this->l('The message length is not proper!'),
                    '7' =>    $this->l('The line is not for send all'),
                    '9' =>    $this->l('You can not send multiple reciver SMS at night time'),
                    '98' => $this->l('The maximum reciver is not alowed!'),
                    '99' => $this->l('The operator is Off!'),
                    '962' =>$this->l('The username or password is incorrect!'),
                    '963' =>$this->l('The user is limited!'),
                    );
                if ($res_code == '0') {
                    $status = $result = 'sent';
                } else {
                    $status = 'failed';
                    if (isset($errorCodes[$res_code])) {
                        $result = $errorCodes[$res_code];
                    } else {
                        $result = $res_data.' - Error Code2:'.$res_code;
                    }
                }
                $result = array('status' => $status,'result' => $result,'res_data' => $res_data);
                break;
        }
        return $result;
    }

    /**
    * Add the CSS & JavaScript files you want to be loaded in the BO.
    */
    public function hookActionAdminControllerSetMedia($params)
    {
        if (Tools::getValue('controller') == 'AdminOrders') { 
            $this->context->controller->addJS($this->_path.'views/js/adminorders.js');
        }
        if (Tools::getValue('controller') == 'AdminAddresses' && !Tools::getValue('id_address')) {
            $this->context->controller->addJS($this->_path.'views/js/adminaddress.js'); 
        }	
        if (Tools::getValue('controller') == 'AdminCustomers' and Tools::getValue('id_customer')) {
            $this->context->controller->addJS($this->_path.'views/js/admincustomers.js');
        } elseif (Tools::getValue('controller') == 'AdminCustomers') {
            $this->context->controller->addJS($this->_path.'views/js/editcustomer.js');
        }
		if (Tools::getValue('controller') == 'AdminModules' and Tools::getValue('configure') == 'ranginesmspresta') {
			$this->context->controller->addCSS($this->_path.'/views/css/backoffice.css', 'all');
			$this->context->controller->addJS($this->_path.'/views/js/backoffice.js', 'all');
		}
		
    }
}
/********TRANSLATION GATE***
$this->l('* You should add a mobile number for admin at Gateway Settings page to recieve admin alerts.')
$this->l('Actions')
$this->l('Admin phone:')
$this->l('Alert on create new account to admin:')
$this->l('Alert on create new account to customer:')
$this->l('Alert on create new address to customer:')
$this->l('Alert on update order status to customer:')
$this->l('Alerts Settings')
$this->l('Alerts on new free order to admin:')
$this->l('Alerts on new free order to customer:')
$this->l('Alerts on new order to Admin:')
$this->l('Alerts on new order to Customer:')
$this->l('Alerts when pruducts goes out of stock:')
$this->l('BULK')
$this->l('By Sample')
$this->l('CUSTOMER')
$this->l('Clear Logs on Uninstall:')
$this->l('Close')
$this->l('Count')
$this->l('Custom Text:')
$this->l('Custom')
$this->l('DELIVERY')
$this->l('DESCRIPTION')
$this->l('Dear {firstname} {lastname},~Welcome to our site.')
$this->l('Dear {firstname} {lastname},~Your order has created.~Order No: {order_id}~{shop_name}')
$this->l('Dear {firstname} {lastname},~Your order has sent by {carrier}~Order: {order_id}~Tracking No: {tracking}~{shop_name}')
$this->l('Enable if you want to clear all SMS logs when you uninstall this module.')
$this->l('Enable if you want you site to send SMS.')
$this->l('Gateway Settings')
$this->l('Gateway alert')
$this->l('Gives the customer the option of receiving a sms notification when an out-of-stock product is available again.')
$this->l('ID')
$this->l('No one has subscribed to any product.')
$this->l('No')
$this->l('On new order sms')
$this->l('On set order tracking number sms')
$this->l('On update order status sms')
$this->l('Out of Stock Subscribers Report')
$this->l('PHONE')
$this->l('POSITION')
$this->l('Password:')
$this->l('Phone List')
$this->l('Product Atribute')
$this->l('Product availability:')
$this->l('Product')
$this->l('Quantity for which a product is considered out of stock.')
$this->l('Rangine SMS login Password.')
$this->l('Remove Subscribers')
$this->l('SHOP')
$this->l('SMS Texts')
$this->l('STATUS')
$this->l('Save')
$this->l('Select which type you want to send to admin on new account event.')
$this->l('Select which type you want to send to admin on new order event.')
$this->l('Select which type you want to send to customer on new account event.')
$this->l('Select which type you want to send to customer on new order event.')
$this->l('Select which type you want to send to customer on update order event.')
$this->l('Send Customers')
$this->l('Send Invoice:')
$this->l('Send SMS Manual')
$this->l('Send SMS')
$this->l('Send a SMS')
$this->l('Send back to stock SMS alerts immediately after a product goes back to stock. The admin recieved a sms in both cases.')
$this->l('Send sms to customers')
$this->l('Send to Admin')
$this->l('Send to All Customers')
$this->l('Send to Customer')
$this->l('Send to customer. If set to False you should go to Out Of Stock tab and send Sms manual.')
$this->l('Sent Back to Stock Automaticly:')
$this->l('Sent SMS Report')
$this->l('Sent to:')
$this->l('Service Sms number:')
$this->l('Settings is not updated!')
$this->l('Settings updated.')
$this->l('Shop')
$this->l('Sms number:')
$this->l('TIME')
$this->l('Text for Admin on New Account:')
$this->l('Text for Admin on New Order:')
$this->l('Text for Admin on out of stock:')
$this->l('Text for Customer on New Account:')
$this->l('Text for Customer on New Address:')
$this->l('Text for Customer on New Order:')
$this->l('Text for Customer on Update Order Tracking:')
$this->l('Text for Customer on Update Order:')
$this->l('Text message:')
$this->l('The sms sent.')
$this->l('The sms sent to customers.')
$this->l('Threshold:')
$this->l('To send multiple sms, saperate numbers with ; ex:09123456789;09101234567.')
$this->l('Type SMS text.')
$this->l('Type your custom text. you can use this variables: {firstname} {lastname}{shop_name}')
$this->l('Update Settings')
$this->l('Username:')
$this->l('Variables: {product_id} {product_name} {shop_name} {product_quantity} {product_reference}')
$this->l('Variables: {shop_name} {firstname} {lastname} {order_id} {order_reference} {carrier}')
$this->l('Variables: {shop_name} {firstname} {lastname} {order_id} {order_reference} {order_state}')
$this->l('Welcome to Rangine SMS module for Prestashop. You can have the best exprience of using SMS module to develope your business.')
$this->l('Welcome')
$this->l('Yes')
$this->l('default')
$this->l('ex: +985000XXXXXXX')
$this->l('ex: +985000XXXXXXX.')
$this->l('ex: 09123456789')
$this->l('ex: username.')
$this->l('insert valid phone number')
$this->l('sent')
$this->l('Text is empty')
$this->l('unique phone mobile(s) from customers is exist')
$this->l('{firstname} {lastname} registred on site')
$this->l('{product_name} goes back to stock!~{product_url}~{shop_name}')
$this->l('{product_id} goes out to stock on {shop_name}.~Quantity: {product_quantity}')
$this->l('{shop_name}~Dear {firstname} {lastname},~New status for order: {order_id}: {order_state}')
$this->l('Enable SMS system:')		
$this->l('Alert on set order tracking number to customer:')
$this->l('Order Mobile Number:')
$this->l('Customer First Address Mobile')
$this->l('Order Address Mobile')
$this->l('Which mobile number should be used for order alerts. If you have guest customer, choose order number')
$this->l('The lisence saved successfully')
$this->l('Error in saving license. Please Try again.')
$this->l('Subscribed Phone List')
$this->l('Number')
$this->l('Customer Numbers')
$this->l('Text for subscribers on back to stock:')
$this->l('{product_name} goes back to stock!~{product_url}~{shop_name}.')
$this->l('New order at {shop_name}~Customer: {firstname} {lastname}~Order: {order_id}~Payment: {payment}~Total: {total_paid} {currency}')
$this->l('You should set your license to achive this page!')
$this->l('Delay Time:')
$this->l('Sender is empty')
$this->l('License is empty')
$this->l('Which number should be used to send this sms. ex: +985000XXXXXXX')
$this->l('When enabled, the checkbox in order status update pane of order edit page checked as default.')
$this->l('When enabled, the checkbox in order tracking number update pane of order edit page checked as default.')
*/